// https://developer.mozilla.org/en-US/docs/Web/API/EventTarget.addEventListener

function closestParentOrSelfAnchorElement(node) {
    while (node) {
        if (node.nodeName.toLowerCase() === 'a') {
            return node;
        }

        node = node.parentNode;
    }

    return null;
}

function isFunction(f) {
    var fDefaultString = Object.prototype.toString.call(f);
    var fIsAFunction = fDefaultString.indexOf('Function') > -1;
    return fIsAFunction;
}

function isString(s) {
    var sDefaultString = Object.prototype.toString.call(s);
    var sIsAString = sDefaultString.indexOf('String') > -1;
    return sIsAString;
}

function consoleLogAndThrow(message) {
    if (console && isFunction(console.log)) {
        console.log(message);
    }

    throw message;
}

function debugToString(object) {
    if ((typeof object) === "undefined") {
        return "undefined";
    } else if (object === null) {
        return "null";
    } else {
        return "(constructor, prototype, typeof, object): " +
            object.constructor + ", " +
            object.prototype + ", " +
            (typeof object) + ", " +
            object;
    }
}

function createAttributesMapForElement(element) {
    var attributesSnapshot, i, attr, attributesMap;
    if (isFunction(document.evaluate) && XPathResult) {
        attributesSnapshot = document.evaluate("./@*", (element), null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
        if ((attributesSnapshot.snapshotLength !== undefined) && isFunction(attributesSnapshot.snapshotItem)) {
            attributesMap = {};
            for (i = 0; i < attributesSnapshot.snapshotLength; i++) {
                attr = attributesSnapshot.snapshotItem(i);
                attributesMap[attr.name] = attr.value;
            }
            return attributesMap;
        } else {
            consoleLogAndThrow("document.evaluate didn't return an ORDERED_NODE_SNAPSHOT_TYPE, but returned " + debugToString(attributesSnapshot));
        }
    } else {
        consoleLogAndThrow("Can't XPath. document.evaluate isn't a function: " + debugToString(document.evaluate) + " or XPathResult doesn't exist: " + debugToString(XPathResult));
    }
}

function attributesMapHasOneDataAttribute(attributesMap) {
    var name, value;
    for (name in attributesMap) {
        if (attributesMap.hasOwnProperty(name)) {
            value = attributesMap[name];
            if (isString(name)) {
                if (name.toLowerCase().indexOf("data-") === 0) {
                    if (isString(value)) {
                        if (value.length > 0) {
                            return true;
                        }
                    }
                }
            }
        }
    }

    return false;
}

function elementHrefPointsToJiveImage(element) {
    var href, hrefPointsToJiveAPIImage;
    if (jiveForAndroid && isFunction(jiveForAndroid.isJiveImageUrl)) {
        href = element.href;
        hrefPointsToJiveAPIImage = jiveForAndroid.isJiveImageUrl(href);
        return hrefPointsToJiveAPIImage;
    } else {
        consoleLogAndThrow("Can't call back to jiveForAndroid.isJiveDesktopImageUrl: " + debugToString(jiveForAndroid));
    }
}

function preventEventPropagation(event) {
// http://www.w3.org/TR/DOM-Level-3-Events/#methods
    if (isFunction(event.preventDefault)) {
        event.preventDefault();
    } else {
        consoleLogAndThrow("Can't call event.preventDefault: " + event);
    }
    // being paranoid since stopImmediatePropagation is only in DOM Level 3.
    // Call both stopImmediatePropagation and stopPropagation just to be sure.
    var stoppedPropagation = false;
    if (isFunction(event.stopPropagation)) {
        event.stopPropagation();
        stoppedPropagation = true;
    }
    if (isFunction(event.stopImmediatePropagation)) {
        event.stopImmediatePropagation();
        stoppedPropagation = true;
    }

    if (!stoppedPropagation) {
        consoleLogAndThrow("Couldn't stopPropagation or stopImmediatePropagation on event: " + event);
    }
}

function callJiveForAndroidOnAnchorClickAndPreventEventDefault(event, anchorElementAttributesMap) {
    var anchorElementAttributesMapJsonString;
    if (JSON && isFunction(JSON.stringify)) {
        anchorElementAttributesMapJsonString = JSON.stringify(anchorElementAttributesMap);
        if (jiveForAndroid && isFunction(jiveForAndroid.onAnchorClick)) {
            preventEventPropagation(event);
            jiveForAndroid.onAnchorClick(anchorElementAttributesMapJsonString);
        } else {
            consoleLogAndThrow("Can't call back to jiveForAndroid.onAnchorClick: " + debugToString(jiveForAndroid));
        }
    } else {
        consoleLogAndThrow("Can't JSON: " + debugToString(JSON));
    }
}

function callJiveForAndroidOnScrollToPositionAndPreventEventDefault(event, position) {
    if (jiveForAndroid && isFunction(jiveForAndroid.onScrollToPosition)) {
        preventEventPropagation(event);
        jiveForAndroid.onScrollToPosition(position);
    } else {
        consoleLogAndThrow("Can't call back to jiveForAndroid.onAnchorClick: " + debugToString(jiveForAndroid));
    }
}

function calculateVerticalPositionOfElementById(id) {
    var y = 0;
    var elem = document.getElementById(id);

    while (elem != null) {
        y += elem.offsetTop;
        elem = elem.offsetParent;
    }

    return y;
}

document.addEventListener("click", function (event) {
    var target, anchorElement, anchorElementAttributesMap, callCallJiveForAndroidOnAnchorClickAndPreventEventDefault;
    target = event.target ? event.target : event.srcElement;
    anchorElement = closestParentOrSelfAnchorElement(target);
    if (anchorElement) {
        anchorElementAttributesMap = createAttributesMapForElement(anchorElement);
        callCallJiveForAndroidOnAnchorClickAndPreventEventDefault = function () {
            callJiveForAndroidOnAnchorClickAndPreventEventDefault(event, anchorElementAttributesMap);
        };
        if (attributesMapHasOneDataAttribute(anchorElementAttributesMap)) {
            callCallJiveForAndroidOnAnchorClickAndPreventEventDefault();
        } else if (elementHrefPointsToJiveImage(anchorElement)) {
            callCallJiveForAndroidOnAnchorClickAndPreventEventDefault();
        } else {
            var href = anchorElementAttributesMap["href"];
            if ( href != null && href[0] == "#") {
                var id = href.substring(1);
                var position = calculateVerticalPositionOfElementById(id);
                callJiveForAndroidOnScrollToPositionAndPreventEventDefault(event, position);
            } else {
                // We have no data attributes and the href isn't an API image nor an anchor.
                // There is nothing for us to do.
            }
        }
    } else {
        // found no anchor element, no worries.
    }
}, false);
